Package: lsirm12pl
Type: Package
Title: Latent Space Item Response Model
Version: 2.0.2
Date: 2026-05-28
Authors@R: c(person("Dongyoung", "Go", role = c("aut"),
                     email = "dongyoung.gr@yonsei.ac.kr"),
	       person("Gwanghee", "Kim", role = c("aut", "cre"),
                     email = "musagh08@yonsei.ac.kr"),
              person("Jina", "Park", role = c("aut"),
                     email = "pja070707@gmail.com"),       
              person("Ickhoon", "Jin", role = c("ctb"),
                     email = "ijin@yonsei.ac.kr"),
              person("Minjeong", "Jeon", role = "ctb",
                     email = "mjjeon@ucla.edu"))
Maintainer: Gwanghee Kim <musagh08@yonsei.ac.kr>
Description: Analysis of dichotomous, ordinal, and continuous response data using latent space item response model ('LSIRM'). Provides 1PL and 2PL 'LSIRM' for binary response data as described in Jeon et al. (2021) <doi:10.1007/s11336-021-09762-5>, graded response models ('GRM') for ordinal data (De Carolis et al., 2025, <doi:10.1080/00273171.2025.2605678>), and extensions for continuous response data. Supports Bayesian model selection with spike-and-slab priors, adaptive MCMC algorithms, and methods for handling missing data under missing at random ('MAR') and missing completely at random ('MCAR') assumptions. Provides various diagnostic plots to inspect the latent space and summaries of estimated parameters.
License: GPL-3
Imports: Rcpp (>= 1.0.5), MCMCpack, ggplot2, GPArotation, dplyr (>=
        1.1.4), grDevices, rlang, pROC, coda, spatstat, spatstat.geom,
        spatstat.random, plotly, gridExtra, grid, tidyr, fpc, kernlab,
        plyr, purrr
LinkingTo: Rcpp, RcppArmadillo
Encoding: UTF-8
RoxygenNote: 7.3.2
Suggests: knitr
Depends: R (>= 4.0.0)
LazyData: true
NeedsCompilation: yes
Packaged: 2026-05-29 14:13:05 UTC; gwangheekim
Author: Dongyoung Go [aut],
  Gwanghee Kim [aut, cre],
  Jina Park [aut],
  Ickhoon Jin [ctb],
  Minjeong Jeon [ctb]
Repository: CRAN
Date/Publication: 2026-05-29 14:40:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-05-29 17:46:36 UTC; unix
Archs: lsirm12pl.so.dSYM
