# Generated by roxygen2: do not edit by hand

S3method(diagnostic,lsirm)
S3method(gof,lsirm)
S3method(lsirm,formula)
S3method(plot,lsirm)
S3method(print,summary.lsirm)
S3method(summary,lsirm)
export(diagnostic)
export(gof)
export(lsirm)
export(lsirm1pl)
export(lsirm1pl_fixed_gamma)
export(lsirm1pl_fixed_gamma_mar)
export(lsirm1pl_fixed_gamma_mcar)
export(lsirm1pl_mar)
export(lsirm1pl_mar_ss)
export(lsirm1pl_mcar)
export(lsirm1pl_mcar_ss)
export(lsirm1pl_normal_fixed_gamma)
export(lsirm1pl_normal_fixed_gamma_mar)
export(lsirm1pl_normal_fixed_gamma_mcar)
export(lsirm1pl_normal_mar)
export(lsirm1pl_normal_mar_ss)
export(lsirm1pl_normal_mcar)
export(lsirm1pl_normal_mcar_ss)
export(lsirm1pl_normal_o)
export(lsirm1pl_normal_ss)
export(lsirm1pl_o)
export(lsirm1pl_ss)
export(lsirm2pl)
export(lsirm2pl_fixed_gamma)
export(lsirm2pl_fixed_gamma_mar)
export(lsirm2pl_fixed_gamma_mcar)
export(lsirm2pl_mar)
export(lsirm2pl_mar_ss)
export(lsirm2pl_mcar)
export(lsirm2pl_mcar_ss)
export(lsirm2pl_normal_fixed_gamma)
export(lsirm2pl_normal_fixed_gamma_mar)
export(lsirm2pl_normal_fixed_gamma_mcar)
export(lsirm2pl_normal_mar)
export(lsirm2pl_normal_mar_ss)
export(lsirm2pl_normal_mcar)
export(lsirm2pl_normal_mcar_ss)
export(lsirm2pl_normal_o)
export(lsirm2pl_normal_ss)
export(lsirm2pl_o)
export(lsirm2pl_ss)
export(lsirmgrm)
export(lsirmgrm2pl)
export(lsirmgrm2pl_fixed_gamma)
export(lsirmgrm2pl_mcar)
export(lsirmgrm2pl_o)
export(lsirmgrm2pl_ss)
export(lsirmgrm_fixed_gamma)
export(lsirmgrm_mcar)
export(lsirmgrm_o)
export(lsirmgrm_ss)
export(onepl)
export(plot)
export(twopl)
import(GPArotation)
import(coda)
import(dplyr)
import(ggplot2)
import(grid)
import(pROC)
import(parallel)
import(purrr)
import(spatstat)
import(tidyr)
importFrom(MCMCpack,procrustes)
importFrom(Rcpp,evalCpp)
importFrom(fpc,cluster.stats)
importFrom(grDevices,boxplot.stats)
importFrom(grDevices,dev.interactive)
importFrom(grDevices,dev.off)
importFrom(grDevices,devAskNewPage)
importFrom(grDevices,pdf)
importFrom(grDevices,rainbow)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,rug)
importFrom(graphics,title)
importFrom(gridExtra,arrangeGrob)
importFrom(gridExtra,grid.arrange)
importFrom(kernlab,specc)
importFrom(plotly,add_markers)
importFrom(plotly,add_text)
importFrom(plotly,ggplotly)
importFrom(plyr,laply)
importFrom(spatstat.geom,owin)
importFrom(spatstat.random,rpoispp)
importFrom(stats,acf)
importFrom(stats,density)
importFrom(stats,dist)
importFrom(stats,kmeans)
importFrom(stats,median)
importFrom(stats,pnorm)
importFrom(stats,printCoefmat)
importFrom(stats,quantile)
importFrom(stats,rbinom)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,setNames)
importFrom(stats,ts.plot)
importFrom(utils,capture.output)
importFrom(utils,setTxtProgressBar)
importFrom(utils,stack)
importFrom(utils,txtProgressBar)
useDynLib(lsirm12pl, .registration = TRUE)
