#' @title Big Five Personality Test
#'
#' @description A dataset containing the result of personality test for 50 questions from 1,000 random sampled people. 
#'
#' @docType data
#' 
#' @usage data(BFPT)
#' @format A matrix with 1,015,341 rows and 50 columns.
#' @details A dataset collected in 2016-2018 through an interactive on-line personality test, containing the result of personality test for 50 questions.
#' 1,000 people are random sampled from the original dataset containing 1,015,341 people. The scale is labeled as 1=Disagree, 3=Neutral and 5=Agree. 
#' @source \url{https://www.kaggle.com/tunguz/big-five-personality-test}
#' 
#' @name BFPT
NULL


#' @title Inductive Reasoning Developmental Test
#'
#' @description TDRI dataset is the answer to Inductive Reasoning Developmental Test of 1,803 Brazilians with age varying from 5 to 85 years.
#' 
#'
#' @docType data
#' @usage data(TDRI)
#' 
#' @format A binary matrix with 1,803 rows and 56 columns.
#' @details It presents data from 1,803 Brazilians (52.5\% female) with age varying from 5 to 85 years (M = 15.75; SD = 12.21) that answered to the Inductive Reasoning Developmental Test – IRDT, with 56 items designed to assess developmentally sequenced and hierarchically organized inductive reasoning.
#' @source \url{https://figshare.com/articles/dataset/TDRI_dataset_csv/3142321}
#' 
#' @name TDRI
NULL


