# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

lsirm1pl_cpp <- function(data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_sd_theta, pr_mean_gamma, pr_sd_gamma, pr_a_theta, pr_b_theta, verbose, fix_theta_sd, adapt = NULL) {
    .Call(`_lsirm12pl_lsirm1pl_cpp`, data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_sd_theta, pr_mean_gamma, pr_sd_gamma, pr_a_theta, pr_b_theta, verbose, fix_theta_sd, adapt)
}

lsirm1pl_mcar_cpp <- function(data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_mean_gamma, pr_sd_gamma, pr_a_theta, pr_b_theta, pr_sd_theta, missing, verbose, fix_theta_sd, fixed_gamma, adapt = NULL) {
    .Call(`_lsirm12pl_lsirm1pl_mcar_cpp`, data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_mean_gamma, pr_sd_gamma, pr_a_theta, pr_b_theta, pr_sd_theta, missing, verbose, fix_theta_sd, fixed_gamma, adapt)
}

lsirm1pl_mcar_ss_cpp <- function(data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_spike_mean, pr_spike_sd, pr_slab_mean, pr_slab_sd, pr_a_theta, pr_b_theta, pr_beta_a, pr_beta_b, pr_sd_theta, missing, verbose, fix_theta_sd, adapt = NULL) {
    .Call(`_lsirm12pl_lsirm1pl_mcar_ss_cpp`, data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_spike_mean, pr_spike_sd, pr_slab_mean, pr_slab_sd, pr_a_theta, pr_b_theta, pr_beta_a, pr_beta_b, pr_sd_theta, missing, verbose, fix_theta_sd, adapt)
}

lsirm1pl_mar_cpp <- function(data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_mean_gamma, pr_sd_gamma, pr_a_theta, pr_b_theta, pr_sd_theta, missing, verbose, fix_theta_sd, fixed_gamma, adapt = NULL) {
    .Call(`_lsirm12pl_lsirm1pl_mar_cpp`, data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_mean_gamma, pr_sd_gamma, pr_a_theta, pr_b_theta, pr_sd_theta, missing, verbose, fix_theta_sd, fixed_gamma, adapt)
}

lsirm1pl_mar_ss_cpp <- function(data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_spike_mean, pr_spike_sd, pr_slab_mean, pr_slab_sd, pr_a_theta, pr_b_theta, pr_beta_a, pr_beta_b, pr_sd_theta, missing, verbose, fix_theta_sd, adapt = NULL) {
    .Call(`_lsirm12pl_lsirm1pl_mar_ss_cpp`, data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_spike_mean, pr_spike_sd, pr_slab_mean, pr_slab_sd, pr_a_theta, pr_b_theta, pr_beta_a, pr_beta_b, pr_sd_theta, missing, verbose, fix_theta_sd, adapt)
}

lsirm1pl_normal_cpp <- function(data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_mean_gamma, pr_sd_gamma, pr_a_theta, pr_b_theta, pr_sd_theta, pr_a_eps, pr_b_eps, verbose, fix_theta_sd, adapt = NULL) {
    .Call(`_lsirm12pl_lsirm1pl_normal_cpp`, data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_mean_gamma, pr_sd_gamma, pr_a_theta, pr_b_theta, pr_sd_theta, pr_a_eps, pr_b_eps, verbose, fix_theta_sd, adapt)
}

lsirm1pl_normal_ss_cpp <- function(data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_spike_mean, pr_spike_sd, pr_slab_mean, pr_slab_sd, pr_a_theta, pr_b_theta, pr_sd_theta, pr_beta_a, pr_beta_b, pr_a_eps, pr_b_eps, verbose, fix_theta_sd, adapt = NULL) {
    .Call(`_lsirm12pl_lsirm1pl_normal_ss_cpp`, data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_spike_mean, pr_spike_sd, pr_slab_mean, pr_slab_sd, pr_a_theta, pr_b_theta, pr_sd_theta, pr_beta_a, pr_beta_b, pr_a_eps, pr_b_eps, verbose, fix_theta_sd, adapt)
}

lsirm1pl_ss_cpp <- function(data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_spike_mean, pr_spike_sd, pr_slab_mean, pr_slab_sd, pr_a_theta, pr_b_theta, pr_beta_a, pr_beta_b, pr_sd_theta, verbose, fix_theta_sd, adapt = NULL) {
    .Call(`_lsirm12pl_lsirm1pl_ss_cpp`, data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_spike_mean, pr_spike_sd, pr_slab_mean, pr_slab_sd, pr_a_theta, pr_b_theta, pr_beta_a, pr_beta_b, pr_sd_theta, verbose, fix_theta_sd, adapt)
}

lsirm1pl_normal_mar_cpp <- function(data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_mean_gamma, pr_sd_gamma, pr_a_theta, pr_b_theta, pr_sd_theta, pr_a_eps, pr_b_eps, missing, verbose, fix_theta_sd, adapt = NULL) {
    .Call(`_lsirm12pl_lsirm1pl_normal_mar_cpp`, data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_mean_gamma, pr_sd_gamma, pr_a_theta, pr_b_theta, pr_sd_theta, pr_a_eps, pr_b_eps, missing, verbose, fix_theta_sd, adapt)
}

lsirm1pl_normal_mcar_ss_cpp <- function(data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_spike_mean, pr_spike_sd, pr_slab_mean, pr_slab_sd, pr_a_theta, pr_b_theta, pr_sd_theta, pr_beta_a, pr_beta_b, pr_a_eps, pr_b_eps, missing, verbose, fix_theta_sd, adapt = NULL) {
    .Call(`_lsirm12pl_lsirm1pl_normal_mcar_ss_cpp`, data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_spike_mean, pr_spike_sd, pr_slab_mean, pr_slab_sd, pr_a_theta, pr_b_theta, pr_sd_theta, pr_beta_a, pr_beta_b, pr_a_eps, pr_b_eps, missing, verbose, fix_theta_sd, adapt)
}

lsirm1pl_normal_mar_ss_cpp <- function(data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_spike_mean, pr_spike_sd, pr_slab_mean, pr_slab_sd, pr_a_theta, pr_b_theta, pr_sd_theta, pr_beta_a, pr_beta_b, pr_a_eps, pr_b_eps, missing, verbose, fix_theta_sd, adapt = NULL) {
    .Call(`_lsirm12pl_lsirm1pl_normal_mar_ss_cpp`, data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_spike_mean, pr_spike_sd, pr_slab_mean, pr_slab_sd, pr_a_theta, pr_b_theta, pr_sd_theta, pr_beta_a, pr_beta_b, pr_a_eps, pr_b_eps, missing, verbose, fix_theta_sd, adapt)
}

lsirm1pl_normal_mcar_cpp <- function(data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_mean_gamma, pr_sd_gamma, pr_a_theta, pr_b_theta, pr_sd_theta, pr_a_eps, pr_b_eps, missing, verbose, fix_theta_sd, adapt = NULL) {
    .Call(`_lsirm12pl_lsirm1pl_normal_mcar_cpp`, data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_mean_gamma, pr_sd_gamma, pr_a_theta, pr_b_theta, pr_sd_theta, pr_a_eps, pr_b_eps, missing, verbose, fix_theta_sd, adapt)
}

lsirm1pl_fixed_gamma_cpp <- function(data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_sd_theta, pr_a_theta, pr_b_theta, verbose, fix_theta_sd, adapt = NULL) {
    .Call(`_lsirm12pl_lsirm1pl_fixed_gamma_cpp`, data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_sd_theta, pr_a_theta, pr_b_theta, verbose, fix_theta_sd, adapt)
}

lsirm1pl_fixed_gamma_mar_cpp <- function(data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_sd_theta, pr_a_theta, pr_b_theta, missing, verbose, fix_theta_sd, adapt = NULL) {
    .Call(`_lsirm12pl_lsirm1pl_fixed_gamma_mar_cpp`, data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_sd_theta, pr_a_theta, pr_b_theta, missing, verbose, fix_theta_sd, adapt)
}

lsirm1pl_fixed_gamma_mcar_cpp <- function(data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_sd_theta, pr_a_theta, pr_b_theta, missing, verbose, fix_theta_sd, adapt = NULL) {
    .Call(`_lsirm12pl_lsirm1pl_fixed_gamma_mcar_cpp`, data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_sd_theta, pr_a_theta, pr_b_theta, missing, verbose, fix_theta_sd, adapt)
}

lsirm1pl_normal_fixed_gamma_cpp <- function(data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_sd_theta, pr_a_theta, pr_b_theta, pr_a_eps, pr_b_eps, verbose, fix_theta_sd, adapt = NULL) {
    .Call(`_lsirm12pl_lsirm1pl_normal_fixed_gamma_cpp`, data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_sd_theta, pr_a_theta, pr_b_theta, pr_a_eps, pr_b_eps, verbose, fix_theta_sd, adapt)
}

lsirm1pl_normal_fixed_gamma_mar_cpp <- function(data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_sd_theta, pr_a_theta, pr_b_theta, pr_a_eps, pr_b_eps, missing, verbose, fix_theta_sd, adapt = NULL) {
    .Call(`_lsirm12pl_lsirm1pl_normal_fixed_gamma_mar_cpp`, data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_sd_theta, pr_a_theta, pr_b_theta, pr_a_eps, pr_b_eps, missing, verbose, fix_theta_sd, adapt)
}

lsirm1pl_normal_fixed_gamma_mcar_cpp <- function(data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_sd_theta, pr_a_theta, pr_b_theta, pr_a_eps, pr_b_eps, missing, verbose, fix_theta_sd, adapt = NULL) {
    .Call(`_lsirm12pl_lsirm1pl_normal_fixed_gamma_mcar_cpp`, data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_sd_theta, pr_a_theta, pr_b_theta, pr_a_eps, pr_b_eps, missing, verbose, fix_theta_sd, adapt)
}

lsirm2pl_cpp <- function(data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_alpha, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_sd_theta, pr_mean_gamma, pr_sd_gamma, pr_mean_alpha, pr_sd_alpha, pr_a_theta, pr_b_theta, verbose, fix_theta_sd, fix_alpha_1, adapt = NULL) {
    .Call(`_lsirm12pl_lsirm2pl_cpp`, data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_alpha, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_sd_theta, pr_mean_gamma, pr_sd_gamma, pr_mean_alpha, pr_sd_alpha, pr_a_theta, pr_b_theta, verbose, fix_theta_sd, fix_alpha_1, adapt)
}

lsirm2pl_mcar_cpp <- function(data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_alpha, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_sd_theta, pr_mean_gamma, pr_sd_gamma, pr_mean_alpha, pr_sd_alpha, pr_a_theta, pr_b_theta, missing, verbose, fix_theta_sd, fix_alpha_1, adapt = NULL) {
    .Call(`_lsirm12pl_lsirm2pl_mcar_cpp`, data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_alpha, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_sd_theta, pr_mean_gamma, pr_sd_gamma, pr_mean_alpha, pr_sd_alpha, pr_a_theta, pr_b_theta, missing, verbose, fix_theta_sd, fix_alpha_1, adapt)
}

lsirm2pl_mcar_ss_cpp <- function(data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_alpha, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_sd_theta, pr_spike_mean, pr_spike_sd, pr_slab_mean, pr_slab_sd, pr_mean_alpha, pr_sd_alpha, pr_a_theta, pr_b_theta, pr_beta_a, pr_beta_b, missing, verbose, fix_theta_sd, fix_alpha_1, adapt = NULL) {
    .Call(`_lsirm12pl_lsirm2pl_mcar_ss_cpp`, data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_alpha, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_sd_theta, pr_spike_mean, pr_spike_sd, pr_slab_mean, pr_slab_sd, pr_mean_alpha, pr_sd_alpha, pr_a_theta, pr_b_theta, pr_beta_a, pr_beta_b, missing, verbose, fix_theta_sd, fix_alpha_1, adapt)
}

lsirm2pl_mar_cpp <- function(data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_alpha, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_sd_theta, pr_mean_gamma, pr_sd_gamma, pr_mean_alpha, pr_sd_alpha, pr_a_theta, pr_b_theta, missing, verbose, fix_theta_sd, fix_alpha_1, adapt = NULL) {
    .Call(`_lsirm12pl_lsirm2pl_mar_cpp`, data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_alpha, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_sd_theta, pr_mean_gamma, pr_sd_gamma, pr_mean_alpha, pr_sd_alpha, pr_a_theta, pr_b_theta, missing, verbose, fix_theta_sd, fix_alpha_1, adapt)
}

lsirm2pl_mar_ss_cpp <- function(data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_alpha, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_sd_theta, pr_spike_mean, pr_spike_sd, pr_slab_mean, pr_slab_sd, pr_mean_alpha, pr_sd_alpha, pr_a_theta, pr_b_theta, pr_beta_a, pr_beta_b, missing, verbose, fix_theta_sd, fix_alpha_1, adapt = NULL) {
    .Call(`_lsirm12pl_lsirm2pl_mar_ss_cpp`, data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_alpha, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_sd_theta, pr_spike_mean, pr_spike_sd, pr_slab_mean, pr_slab_sd, pr_mean_alpha, pr_sd_alpha, pr_a_theta, pr_b_theta, pr_beta_a, pr_beta_b, missing, verbose, fix_theta_sd, fix_alpha_1, adapt)
}

lsirm2pl_normal_cpp <- function(data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_alpha, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_sd_theta, pr_a_theta, pr_b_theta, pr_a_eps, pr_b_eps, pr_mean_gamma, pr_sd_gamma, pr_mean_alpha, pr_sd_alpha, verbose, fix_theta_sd, fix_alpha_1, adapt = NULL) {
    .Call(`_lsirm12pl_lsirm2pl_normal_cpp`, data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_alpha, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_sd_theta, pr_a_theta, pr_b_theta, pr_a_eps, pr_b_eps, pr_mean_gamma, pr_sd_gamma, pr_mean_alpha, pr_sd_alpha, verbose, fix_theta_sd, fix_alpha_1, adapt)
}

lsirm2pl_normal_ss_cpp <- function(data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_alpha, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_sd_theta, pr_spike_mean, pr_spike_sd, pr_slab_mean, pr_slab_sd, pr_mean_alpha, pr_sd_alpha, pr_a_eps, pr_b_eps, pr_a_theta, pr_b_theta, pr_beta_a, pr_beta_b, verbose, fix_theta_sd, fix_alpha_1, adapt = NULL) {
    .Call(`_lsirm12pl_lsirm2pl_normal_ss_cpp`, data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_alpha, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_sd_theta, pr_spike_mean, pr_spike_sd, pr_slab_mean, pr_slab_sd, pr_mean_alpha, pr_sd_alpha, pr_a_eps, pr_b_eps, pr_a_theta, pr_b_theta, pr_beta_a, pr_beta_b, verbose, fix_theta_sd, fix_alpha_1, adapt)
}

lsirm2pl_fixed_gamma_cpp <- function(data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_alpha, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_sd_theta, pr_mean_alpha, pr_sd_alpha, pr_a_theta, pr_b_theta, verbose, fix_theta_sd, fix_alpha_1, adapt = NULL) {
    .Call(`_lsirm12pl_lsirm2pl_fixed_gamma_cpp`, data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_alpha, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_sd_theta, pr_mean_alpha, pr_sd_alpha, pr_a_theta, pr_b_theta, verbose, fix_theta_sd, fix_alpha_1, adapt)
}

lsirm2pl_ss_cpp <- function(data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_alpha, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_sd_theta, pr_spike_mean, pr_spike_sd, pr_slab_mean, pr_slab_sd, pr_mean_alpha, pr_sd_alpha, pr_a_theta, pr_b_theta, pr_beta_a, pr_beta_b, verbose, fix_theta_sd, fix_alpha_1, adapt = NULL) {
    .Call(`_lsirm12pl_lsirm2pl_ss_cpp`, data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_alpha, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_sd_theta, pr_spike_mean, pr_spike_sd, pr_slab_mean, pr_slab_sd, pr_mean_alpha, pr_sd_alpha, pr_a_theta, pr_b_theta, pr_beta_a, pr_beta_b, verbose, fix_theta_sd, fix_alpha_1, adapt)
}

lsirm2pl_normal_fixed_gamma_cpp <- function(data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_alpha, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_sd_theta, pr_a_theta, pr_b_theta, pr_a_eps, pr_b_eps, pr_mean_alpha, pr_sd_alpha, verbose, fix_theta_sd, fix_alpha_1, adapt = NULL) {
    .Call(`_lsirm12pl_lsirm2pl_normal_fixed_gamma_cpp`, data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_alpha, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_sd_theta, pr_a_theta, pr_b_theta, pr_a_eps, pr_b_eps, pr_mean_alpha, pr_sd_alpha, verbose, fix_theta_sd, fix_alpha_1, adapt)
}

lsirm2pl_normal_mar_ss_cpp <- function(data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_alpha, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_sd_theta, pr_spike_mean, pr_spike_sd, pr_slab_mean, pr_slab_sd, pr_mean_alpha, pr_sd_alpha, pr_a_theta, pr_b_theta, pr_beta_a, pr_beta_b, pr_a_eps, pr_b_eps, missing, verbose, fix_theta_sd, fix_alpha_1, adapt = NULL) {
    .Call(`_lsirm12pl_lsirm2pl_normal_mar_ss_cpp`, data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_alpha, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_sd_theta, pr_spike_mean, pr_spike_sd, pr_slab_mean, pr_slab_sd, pr_mean_alpha, pr_sd_alpha, pr_a_theta, pr_b_theta, pr_beta_a, pr_beta_b, pr_a_eps, pr_b_eps, missing, verbose, fix_theta_sd, fix_alpha_1, adapt)
}

lsirm2pl_normal_mcar_ss_cpp <- function(data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_alpha, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_sd_theta, pr_spike_mean, pr_spike_sd, pr_slab_mean, pr_slab_sd, pr_mean_alpha, pr_sd_alpha, pr_a_theta, pr_b_theta, pr_beta_a, pr_beta_b, pr_a_eps, pr_b_eps, missing, verbose, fix_theta_sd, fix_alpha_1, adapt = NULL) {
    .Call(`_lsirm12pl_lsirm2pl_normal_mcar_ss_cpp`, data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_alpha, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_sd_theta, pr_spike_mean, pr_spike_sd, pr_slab_mean, pr_slab_sd, pr_mean_alpha, pr_sd_alpha, pr_a_theta, pr_b_theta, pr_beta_a, pr_beta_b, pr_a_eps, pr_b_eps, missing, verbose, fix_theta_sd, fix_alpha_1, adapt)
}

lsirm2pl_normal_mar_cpp <- function(data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_alpha, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_sd_theta, pr_mean_gamma, pr_sd_gamma, pr_mean_alpha, pr_sd_alpha, pr_a_theta, pr_b_theta, pr_a_eps, pr_b_eps, missing, verbose, fix_theta_sd, fix_alpha_1, adapt = NULL) {
    .Call(`_lsirm12pl_lsirm2pl_normal_mar_cpp`, data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_alpha, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_sd_theta, pr_mean_gamma, pr_sd_gamma, pr_mean_alpha, pr_sd_alpha, pr_a_theta, pr_b_theta, pr_a_eps, pr_b_eps, missing, verbose, fix_theta_sd, fix_alpha_1, adapt)
}

lsirm2pl_normal_mcar_cpp <- function(data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_alpha, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_sd_theta, pr_mean_gamma, pr_sd_gamma, pr_mean_alpha, pr_sd_alpha, pr_a_theta, pr_b_theta, pr_a_eps, pr_b_eps, missing, verbose, fix_theta_sd, fix_alpha_1, adapt = NULL) {
    .Call(`_lsirm12pl_lsirm2pl_normal_mcar_cpp`, data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_alpha, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_sd_theta, pr_mean_gamma, pr_sd_gamma, pr_mean_alpha, pr_sd_alpha, pr_a_theta, pr_b_theta, pr_a_eps, pr_b_eps, missing, verbose, fix_theta_sd, fix_alpha_1, adapt)
}

lsirm2pl_fixed_gamma_mcar_cpp <- function(data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_alpha, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_sd_theta, pr_mean_alpha, pr_sd_alpha, pr_a_theta, pr_b_theta, missing, verbose, fix_theta_sd, fix_alpha_1, adapt = NULL) {
    .Call(`_lsirm12pl_lsirm2pl_fixed_gamma_mcar_cpp`, data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_alpha, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_sd_theta, pr_mean_alpha, pr_sd_alpha, pr_a_theta, pr_b_theta, missing, verbose, fix_theta_sd, fix_alpha_1, adapt)
}

lsirm2pl_fixed_gamma_mar_cpp <- function(data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_alpha, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_sd_theta, pr_mean_alpha, pr_sd_alpha, pr_a_theta, pr_b_theta, missing, verbose, fix_theta_sd, fix_alpha_1, adapt = NULL) {
    .Call(`_lsirm12pl_lsirm2pl_fixed_gamma_mar_cpp`, data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_alpha, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_sd_theta, pr_mean_alpha, pr_sd_alpha, pr_a_theta, pr_b_theta, missing, verbose, fix_theta_sd, fix_alpha_1, adapt)
}

lsirm2pl_normal_fixed_gamma_mcar_cpp <- function(data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_alpha, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_sd_theta, pr_mean_alpha, pr_sd_alpha, pr_a_theta, pr_b_theta, pr_a_eps, pr_b_eps, missing, verbose, fix_theta_sd, fix_alpha_1, adapt = NULL) {
    .Call(`_lsirm12pl_lsirm2pl_normal_fixed_gamma_mcar_cpp`, data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_alpha, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_sd_theta, pr_mean_alpha, pr_sd_alpha, pr_a_theta, pr_b_theta, pr_a_eps, pr_b_eps, missing, verbose, fix_theta_sd, fix_alpha_1, adapt)
}

lsirm2pl_normal_fixed_gamma_mar_cpp <- function(data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_alpha, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_sd_theta, pr_mean_alpha, pr_sd_alpha, pr_a_theta, pr_b_theta, pr_a_eps, pr_b_eps, missing, verbose, fix_theta_sd, fix_alpha_1, adapt = NULL) {
    .Call(`_lsirm12pl_lsirm2pl_normal_fixed_gamma_mar_cpp`, data, ndim, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_alpha, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_sd_theta, pr_mean_alpha, pr_sd_alpha, pr_a_theta, pr_b_theta, pr_a_eps, pr_b_eps, missing, verbose, fix_theta_sd, fix_alpha_1, adapt)
}

lsirmgrm_cpp <- function(data, ndim, ncat, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_a_theta, pr_b_theta, pr_mean_theta, pr_sd_theta, pr_mean_gamma, pr_sd_gamma, jump_gamma, missing, verbose, fix_theta_sd, adapt = NULL) {
    .Call(`_lsirm12pl_lsirmgrm_cpp`, data, ndim, ncat, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_a_theta, pr_b_theta, pr_mean_theta, pr_sd_theta, pr_mean_gamma, pr_sd_gamma, jump_gamma, missing, verbose, fix_theta_sd, adapt)
}

lsirmgrm_fixed_gamma_cpp <- function(data, ndim, ncat, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_a_theta, pr_b_theta, pr_mean_theta, pr_sd_theta, missing, verbose, fix_theta_sd, adapt = NULL) {
    .Call(`_lsirm12pl_lsirmgrm_fixed_gamma_cpp`, data, ndim, ncat, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_a_theta, pr_b_theta, pr_mean_theta, pr_sd_theta, missing, verbose, fix_theta_sd, adapt)
}

lsirmgrm_ss_cpp <- function(data, ndim, ncat, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_a_theta, pr_b_theta, pr_mean_theta, pr_sd_theta, pr_spike_mean, pr_spike_sd, pr_slab_mean, pr_slab_sd, pr_beta_a, pr_beta_b, missing, verbose, fix_theta_sd, adapt = NULL) {
    .Call(`_lsirm12pl_lsirmgrm_ss_cpp`, data, ndim, ncat, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_a_theta, pr_b_theta, pr_mean_theta, pr_sd_theta, pr_spike_mean, pr_spike_sd, pr_slab_mean, pr_slab_sd, pr_beta_a, pr_beta_b, missing, verbose, fix_theta_sd, adapt)
}

lsirmgrm_mar_cpp <- function(data, ndim, ncat, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_a_theta, pr_b_theta, pr_mean_theta, pr_sd_theta, pr_mean_gamma, pr_sd_gamma, jump_gamma, missing, verbose, fix_theta_sd, adapt = NULL) {
    .Call(`_lsirm12pl_lsirmgrm_mar_cpp`, data, ndim, ncat, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_a_theta, pr_b_theta, pr_mean_theta, pr_sd_theta, pr_mean_gamma, pr_sd_gamma, jump_gamma, missing, verbose, fix_theta_sd, adapt)
}

lsirmgrm_fixed_gamma_mar_cpp <- function(data, ndim, ncat, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_a_theta, pr_b_theta, pr_mean_theta, pr_sd_theta, missing, verbose, fix_theta_sd, adapt = NULL) {
    .Call(`_lsirm12pl_lsirmgrm_fixed_gamma_mar_cpp`, data, ndim, ncat, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_a_theta, pr_b_theta, pr_mean_theta, pr_sd_theta, missing, verbose, fix_theta_sd, adapt)
}

lsirmgrm_ss_mar_cpp <- function(data, ndim, ncat, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_a_theta, pr_b_theta, pr_mean_theta, pr_sd_theta, pr_spike_mean, pr_spike_sd, pr_slab_mean, pr_slab_sd, pr_beta_a, pr_beta_b, missing, verbose, fix_theta_sd, adapt = NULL) {
    .Call(`_lsirm12pl_lsirmgrm_ss_mar_cpp`, data, ndim, ncat, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_a_theta, pr_b_theta, pr_mean_theta, pr_sd_theta, pr_spike_mean, pr_spike_sd, pr_slab_mean, pr_slab_sd, pr_beta_a, pr_beta_b, missing, verbose, fix_theta_sd, adapt)
}

lsirmgrm2pl_cpp <- function(data, ndim, ncat, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_alpha, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_a_theta, pr_b_theta, pr_mean_theta, pr_sd_theta, pr_mean_gamma, pr_sd_gamma, jump_gamma, pr_mean_alpha, pr_sd_alpha, missing, verbose, fix_theta_sd, fix_alpha_1, adapt = NULL) {
    .Call(`_lsirm12pl_lsirmgrm2pl_cpp`, data, ndim, ncat, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_alpha, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_a_theta, pr_b_theta, pr_mean_theta, pr_sd_theta, pr_mean_gamma, pr_sd_gamma, jump_gamma, pr_mean_alpha, pr_sd_alpha, missing, verbose, fix_theta_sd, fix_alpha_1, adapt)
}

lsirmgrm2pl_fixed_gamma_cpp <- function(data, ndim, ncat, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_alpha, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_a_theta, pr_b_theta, pr_mean_theta, pr_sd_theta, pr_mean_alpha, pr_sd_alpha, missing, verbose, fix_theta_sd, fix_alpha_1, adapt = NULL) {
    .Call(`_lsirm12pl_lsirmgrm2pl_fixed_gamma_cpp`, data, ndim, ncat, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_alpha, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_a_theta, pr_b_theta, pr_mean_theta, pr_sd_theta, pr_mean_alpha, pr_sd_alpha, missing, verbose, fix_theta_sd, fix_alpha_1, adapt)
}

lsirmgrm2pl_ss_cpp <- function(data, ndim, ncat, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_alpha, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_a_theta, pr_b_theta, pr_mean_theta, pr_sd_theta, pr_spike_mean, pr_spike_sd, pr_slab_mean, pr_slab_sd, pr_beta_a, pr_beta_b, pr_mean_alpha, pr_sd_alpha, missing, verbose, fix_theta_sd, fix_alpha_1, adapt = NULL) {
    .Call(`_lsirm12pl_lsirmgrm2pl_ss_cpp`, data, ndim, ncat, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_alpha, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_a_theta, pr_b_theta, pr_mean_theta, pr_sd_theta, pr_spike_mean, pr_spike_sd, pr_slab_mean, pr_slab_sd, pr_beta_a, pr_beta_b, pr_mean_alpha, pr_sd_alpha, missing, verbose, fix_theta_sd, fix_alpha_1, adapt)
}

lsirmgrm2pl_mar_cpp <- function(data, ndim, ncat, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_alpha, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_a_theta, pr_b_theta, pr_mean_theta, pr_sd_theta, pr_mean_gamma, pr_sd_gamma, jump_gamma, pr_mean_alpha, pr_sd_alpha, missing, verbose, fix_theta_sd, fix_alpha_1, adapt = NULL) {
    .Call(`_lsirm12pl_lsirmgrm2pl_mar_cpp`, data, ndim, ncat, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_alpha, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_a_theta, pr_b_theta, pr_mean_theta, pr_sd_theta, pr_mean_gamma, pr_sd_gamma, jump_gamma, pr_mean_alpha, pr_sd_alpha, missing, verbose, fix_theta_sd, fix_alpha_1, adapt)
}

lsirmgrm2pl_fixed_gamma_mar_cpp <- function(data, ndim, ncat, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_alpha, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_a_theta, pr_b_theta, pr_mean_theta, pr_sd_theta, pr_mean_alpha, pr_sd_alpha, missing, verbose, fix_theta_sd, fix_alpha_1, adapt = NULL) {
    .Call(`_lsirm12pl_lsirmgrm2pl_fixed_gamma_mar_cpp`, data, ndim, ncat, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_alpha, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_a_theta, pr_b_theta, pr_mean_theta, pr_sd_theta, pr_mean_alpha, pr_sd_alpha, missing, verbose, fix_theta_sd, fix_alpha_1, adapt)
}

lsirmgrm2pl_ss_mar_cpp <- function(data, ndim, ncat, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_alpha, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_a_theta, pr_b_theta, pr_mean_theta, pr_sd_theta, pr_spike_mean, pr_spike_sd, pr_slab_mean, pr_slab_sd, pr_beta_a, pr_beta_b, pr_mean_alpha, pr_sd_alpha, missing, verbose, fix_theta_sd, fix_alpha_1, adapt = NULL) {
    .Call(`_lsirm12pl_lsirmgrm2pl_ss_mar_cpp`, data, ndim, ncat, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_alpha, jump_gamma, jump_z, jump_w, pr_mean_beta, pr_sd_beta, pr_a_theta, pr_b_theta, pr_mean_theta, pr_sd_theta, pr_spike_mean, pr_spike_sd, pr_slab_mean, pr_slab_sd, pr_beta_a, pr_beta_b, pr_mean_alpha, pr_sd_alpha, missing, verbose, fix_theta_sd, fix_alpha_1, adapt)
}

onepl_cpp <- function(data, niter, nburn, nthin, nprint, jump_beta, jump_theta, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_a_theta, pr_b_theta) {
    .Call(`_lsirm12pl_onepl_cpp`, data, niter, nburn, nthin, nprint, jump_beta, jump_theta, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_a_theta, pr_b_theta)
}

two_pl <- function(data, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_alpha, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_mean_alpha, pr_sd_alpha, pr_a_theta, pr_b_theta) {
    .Call(`_lsirm12pl_two_pl`, data, niter, nburn, nthin, nprint, jump_beta, jump_theta, jump_alpha, pr_mean_beta, pr_sd_beta, pr_mean_theta, pr_mean_alpha, pr_sd_alpha, pr_a_theta, pr_b_theta)
}

Kumulavsech <- function(CC, omega, xlim, ylim) {
    .Call(`_lsirm12pl_Kumulavsech`, CC, omega, xlim, ylim)
}

logpXCbeta <- function(X, CC, alpha, omega, AreaW, integral) {
    .Call(`_lsirm12pl_logpXCbeta`, X, CC, alpha, omega, AreaW, integral)
}

NewPoint <- function(xlim, ylim) {
    .Call(`_lsirm12pl_NewPoint`, xlim, ylim)
}

log_likelihood_cpp <- function(data, ndim, beta_est, theta_est, gamma_est, z_est, w_est, missing) {
    .Call(`_lsirm12pl_log_likelihood_cpp`, data, ndim, beta_est, theta_est, gamma_est, z_est, w_est, missing)
}

log_likelihood_grm_cpp <- function(data, ndim, ncat, beta_est, theta_est, gamma_est, z_est, w_est, missing) {
    .Call(`_lsirm12pl_log_likelihood_grm_cpp`, data, ndim, ncat, beta_est, theta_est, gamma_est, z_est, w_est, missing)
}

log_likelihood_grm2pl_cpp <- function(data, ndim, ncat, beta_est, alpha_est, theta_est, gamma_est, z_est, w_est, missing) {
    .Call(`_lsirm12pl_log_likelihood_grm2pl_cpp`, data, ndim, ncat, beta_est, alpha_est, theta_est, gamma_est, z_est, w_est, missing)
}

log_likelihood_normal_cpp <- function(data, ndim, beta_est, theta_est, gamma_est, z_est, w_est, sigma_est, missing) {
    .Call(`_lsirm12pl_log_likelihood_normal_cpp`, data, ndim, beta_est, theta_est, gamma_est, z_est, w_est, sigma_est, missing)
}

log_likelihood_normal2pl_cpp <- function(data, ndim, beta_est, alpha_est, theta_est, gamma_est, z_est, w_est, sigma_est, missing) {
    .Call(`_lsirm12pl_log_likelihood_normal2pl_cpp`, data, ndim, beta_est, alpha_est, theta_est, gamma_est, z_est, w_est, sigma_est, missing)
}

log_likelihood_2pl_cpp <- function(data, ndim, beta_est, alpha_est, theta_est, gamma_est, z_est, w_est, missing) {
    .Call(`_lsirm12pl_log_likelihood_2pl_cpp`, data, ndim, beta_est, alpha_est, theta_est, gamma_est, z_est, w_est, missing)
}

