% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lsirm.R
\name{lsirm.formula}
\alias{lsirm.formula}
\title{Formula function for LSIRM}
\usage{
\method{lsirm}{formula}(formula, ...)
}
\arguments{
\item{formula}{The form of formula is \code{lsirm(A ~ <term 1>(<term 2>, <term 3> ...))}, where \code{A} is an item response matrix to be analyzed, \code{<term1>} is the model you want to fit and has one of the following values: "lsirm1pl", "lsirm2pl", "lsirmgrm", and "lsirmgrm2pl"., and \code{<term 2>}, \code{<term 3>}, etc., are each option for the model.}

\item{...}{Additional arguments for the corresponding function.}
}
\description{
\link{lsirm.formula} is formula object.
}
