% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{TDRI}
\alias{TDRI}
\title{Inductive Reasoning Developmental Test}
\format{
A binary matrix with 1,803 rows and 56 columns.
}
\source{
\url{https://figshare.com/articles/dataset/TDRI_dataset_csv/3142321}
}
\usage{
data(TDRI)
}
\description{
TDRI dataset is the answer to Inductive Reasoning Developmental Test of 1,803 Brazilians with age varying from 5 to 85 years.
}
\details{
It presents data from 1,803 Brazilians (52.5\% female) with age varying from 5 to 85 years (M = 15.75; SD = 12.21) that answered to the Inductive Reasoning Developmental Test – IRDT, with 56 items designed to assess developmentally sequenced and hierarchically organized inductive reasoning.
}
