% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{BFPT}
\alias{BFPT}
\title{Big Five Personality Test}
\format{
A matrix with 1,015,341 rows and 50 columns.
}
\source{
\url{https://www.kaggle.com/tunguz/big-five-personality-test}
}
\usage{
data(BFPT)
}
\description{
A dataset containing the result of personality test for 50 questions from 1,000 random sampled people.
}
\details{
A dataset collected in 2016-2018 through an interactive on-line personality test, containing the result of personality test for 50 questions.
1,000 people are random sampled from the original dataset containing 1,015,341 people. The scale is labeled as 1=Disagree, 3=Neutral and 5=Agree.
}
